var log
if(location.protocol=="chrome:") {
    log = function(message) {
        var consoleService = Components.classes['@mozilla.org/consoleservice;1']
                .getService(Components.interfaces.nsIConsoleService);
        consoleService.logStringMessage(message)
    }
} else {
    log = function(message) {
        debug(message)
    }
}

log("loading window.js")

var currentDocument = null

function onLoad() {
    log("onLoad() called");

    loadXPathForNode(opener.xpathTarget)

    document.getElementById("xpath").addEventListener('command',search,false);
    document.getElementById("xpath").addEventListener('input',onTextChangeInXPathBox,false);
}

function loadXPathForNode(node) {
    log("getXPathForNode called")

    document.getElementById("xpath").value = getXPath(node)
    var newUrl = node.ownerDocument.documentURI
    if(newUrl!=getUrl()) {
        loadIFrame(node.ownerDocument)
        // workaround because iframe.onload doesn't work
        setTimeout(search, 1000)
    } else {
        search();
    }
}

function getUrl() {
    var iframe = document.getElementById("resultsFrame")
    return iframe.contentDocument.location.href
}

function loadIFrame(newDocument) {
    var url = newDocument.documentURI
    currentDocument = newDocument

    log("loading iframe from "+url)

    var iframe = document.getElementById("resultsFrame")
    var docShell = iframe.docShell

    docShell.allowAuth = false
    docShell.allowJavascript = false
    docShell.allowMetaRedirects = false
    docShell.allowPlugins = false
    docShell.allowSubframes = false

    document.getElementById("resultsDeck").selectedIndex=0
    iframe.contentDocument.location.replace(url)
    document.getElementById("results-caption").label = "Results from "+url
}

function onTextChangeInXPathBox() {
    var xpath = document.getElementById("xpath").value
    document.getElementById("status").value = isValidXPath(xpath) ? "" : "Syntax error"
}

function search() {
    log("search called")

    var xpath = document.getElementById("xpath").value
    if(!isValidXPath(xpath)) return

    var resultList = getXPathNodes(currentDocument, xpath)

    updateStatus(resultList)
    if(resultList.length>0 &&
       resultList[0].nodeType==Node.ELEMENT_NODE) {
        if(currentDocument instanceof HTMLDocument) {
            updateHtmlResults(resultList)
        } else {
            updateTextResults(serializeXML(resultList))
        }
    } else {
        updateTextResults(getNodeValues(resultList))
    }
}

function updateStatus(results) {
    var status;
    if(results.length==0) {
      status = "No matches found"
    } else if(results.length==1) {
      status = "One match found"
    } else if(results.length>1) {
      status = results.length+" matches found"
    }
    document.getElementById("status").value = status
}

function getNodeValues(resultList) {
    var result = []
    for (var i in resultList) {
        result.push(resultList[i].nodeValue)
    }
    return result
}

function serializeXML(resultList) {
    var serializer = new XMLSerializer()
    var result = []
    for (var i in resultList) {
        result.push(serializer.serializeToString(resultList[i]))
    }
    return result
}

function updateTextResults(items) {
    var newRows = document.createElement("rows")
    newRows.setAttribute("flex","1")
    newRows.setAttribute("style","overflow: auto")
    newRows.setAttribute("id","resultrows")

    for (var i in items) {
        var label = document.createElement("label")
        label.setAttribute("value", (parseInt(i)+1)+":")

        var content = makeTextBox(items[i])

        var row = document.createElement("row")
        row.setAttribute("flex",1)
        row.appendChild(label)
        row.appendChild(content)
        newRows.appendChild(row)
    }

    var oldRows = document.getElementById("resultrows")
    oldRows.parentNode.replaceChild(newRows, oldRows)
    document.getElementById("resultsDeck").selectedIndex=1
}

function makeTextBox(content) {
    var textbox = document.createElement("textbox")
    textbox.setAttribute("flex",1)
    textbox.setAttribute("value",content)
    textbox.setAttribute("readonly",true)
    textbox.setAttribute("multiline",true)
    textbox.setAttribute("minheight","20")
    return textbox
}

function updateHtmlResults(results) {
    var doc = document.getElementById("resultsFrame").contentDocument
    doc.body.innerHTML = "<table><tbody></tbody></table>"

    for (var i in results) {
        var node = results[i]
        var label = (parseInt(i)+1)+":"

        var row = doc.createElement("tr")
        row.appendChild(doc.createElement("td"))
        row.appendChild(doc.createElement("td"))
        row.firstChild.innerHTML = label
        row.lastChild.appendChild(node.cloneNode(true))

        doc.body.lastChild.lastChild.appendChild(row)
    }
    document.getElementById("resultsDeck").selectedIndex=2
}

// ================================================================

function isValidXPath(xpath) {
    var evaluator = new XPathEvaluator()
    try {
        evaluator.createExpression(xpath, null)
    } catch(e) {
        return false;
    }

    if(xpath=='/' || xpath=='.') return false;

    return true;
}

function getXPathNodes(document, xpath) {
    var xpathResult = document.evaluate(xpath, document, null, XPathResult.ANY_TYPE, null);
    var result = [];
    var item = xpathResult.iterateNext();
    while (item != null) {
        result.push(item);
        item = xpathResult.iterateNext();
    }
    return result;
}

function getXPath(targetNode) {
    var useLowerCase = (targetNode.ownerDocument instanceof HTMLDocument)
    var nodePath = getNodePath(targetNode)
    var nodeNames = []
    for (var i in nodePath) {
        var nodeIndex
        var node = nodePath[i]
        if (node.nodeType == 1) { // && node.tagName != "TBODY") {
            if (i == 0 && node.hasAttribute("id")) {
                nodeNames.push("/*[@id='" + node.getAttribute("id") + "']");
            } else {
                var tagName = node.tagName
                if (useLowerCase) {
                    tagName = tagName.toLowerCase()
                }
                nodeIndex = getNodeIndex(node)
                if (nodeIndex != null) {
                    nodeNames.push(tagName + "[" + nodeIndex + "]");
                } else {
                    nodeNames.push(tagName);
                }
            }
        } else if (node.nodeType == 3) {
            nodeIndex = getTextNodeIndex(node)
            if (nodeIndex != null) {
                nodeNames.push("text()[" + nodeIndex + "]");
            } else {
                nodeNames.push("text()");
            }
        }
    }
    return "/" + nodeNames.join("/")

}

function getNodeIndex(node) {
    if (node.nodeType != 1 || node.parentNode == null) return null
    var list = getChildNodesWithTagName(node.parentNode, node.tagName)
    if (list.length == 1 && list[0] == node) return null
    for (var i = 0; i < list.length; i++) {
        if (list[i] == node) return i + 1
    }
    throw "couldn't find node in parent's list: " + node.tagName
}

function getTextNodeIndex(node) {
    var list = getChildTextNodes(node.parentNode)
    if (list.length == 1 && list[0] == node) return null
    for (var i = 0; i < list.length; i++) {
        if (list[i] == node) return i + 1
    }
    throw "couldn't find node in parent's list: " + node.tagName
}

function getChildNodesWithTagName(parent, tagName) {
    var result = []
    var child = parent.firstChild
    while (child != null) {
        if (child.tagName && child.tagName == tagName) {
            result.push(child)
        }
        child = child.nextSibling
    }
    return result
}

function getChildTextNodes(parent) {
    var result = []
    var child = parent.firstChild
    while (child != null) {
        if (child.nodeType==3) {
            result.push(child)
        }
        child = child.nextSibling
    }
    return result
}

function getNodePath(node) {
    var result = []
    while (node.nodeType == 1 || node.nodeType == 3) {
        result.unshift(node)
        if (node.nodeType == 1 && node.hasAttribute("id")) return result
        node = node.parentNode
    }
    return result
}





